package org.objectweb.telosys.auth;

import java.io.IOException;
import java.util.Date;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.env.ScreenSessionManager;
import org.objectweb.telosys.util.StrUtil;

/**
 * Filter to restrict access for authenticated users.
 * Authenticated users can pass through the filter
 * Authentication based on "Basic Http Authentication" but with dynamic Realm Names and URLs
 * to avoid the "Login/password" in each request and to provide a "logout" mechanism  
 * 
 * @author Laurent GUERIN 
 * 
 */ 
public class AuthFilter extends TelosysObject implements Filter
{
	private final static String PATH_LOGIN  = "/l/o/g/i/n/" ;
	private final static String COUNT_PARAM = "c" ;
	private final static String TOKEN_PREFIX = "T" ;

//	private String _sLoginUrl = null ; 

	/**
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig config) throws ServletException
	{
//		String sParam = config.getInitParameter("LoginUrl");
//		if ( sParam != null )
//		{
//			_sLoginUrl = sParam ;
//		}
//		info("init() : Authentication filter initialized ( LoginUrl parameter = '" + _sLoginUrl + "' )");
		info("init() : Authentication filter initialized.");
	}

	/**
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy()
	{
		trace("destroy()");
	}

	/**
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	public void doFilter(final ServletRequest request, final ServletResponse response, final FilterChain chain) throws IOException, ServletException
	{
		trace("----- doFilter()...");
		HttpServletRequest  httpRequest  = null ;
		HttpServletResponse httpResponse = null;
		if ( request instanceof HttpServletRequest && response instanceof HttpServletResponse )
		{
			httpRequest  = (HttpServletRequest) request;
			httpResponse = (HttpServletResponse) response;
		}
		else
		{
		    error("doFilter() : request and/or response not HTTP ");
		    return;
		}

		trace("request.getAuthType()   = " + httpRequest.getAuthType() );		
		trace("request.getRemoteUser() = " + httpRequest.getRemoteUser() );		
		
		//--- Check if user is in session ( already authenticated )  
		if ( Auth.isAuthenticated(httpRequest) ) 
		{			
			//--- For DEBUG 
	        LoginUser user = ScreenSessionManager.findScreenSessionUser(httpRequest);
			trace("User authenticated (found in session) : " + user.getLogin() + " --> OK." );	

			//--- Authenticated user => access permitted => chain
			chain.doFilter(request, response);
		}
		else
		{
			//--- Not authenticated user : => Force authentication ( BASIC or with login URL )
			trace("User not yet authenticated (not found in session).");
			String sAuthURI = httpRequest.getContextPath() + PATH_LOGIN ; // "/context/l/o/g/i/n"
			if ( isLoginURL(httpRequest) ) 			    
			{
				trace("Request URL is a Login URL : try to authenticate the user... ");
			    LoginUser loginUser = getUserFromRequest(httpRequest);
		        if (loginUser != null) // "login/password" found in the Request Header 
		        {
		            //IAppUser appUser = checkUserAuthentication(loginUser);
		            IAppUser appUser = Auth.validUser(loginUser);
		            if ( appUser != null )
		            {
			            trace("User login/password is valid. ");
			            //--- OPEN a SCREEN SESSION for this user
			            trace("Init user in session... ");
			            ScreenSessionManager.openScreenSession(httpRequest, loginUser, appUser);
			            trace("Redirect to first page... ");
					    redirectToFirstPage(httpRequest, httpResponse, appUser );
		            }
		            else
		            {
			            trace("Invalid login/password => '401' (Unauthorized) ");
			            //--- Response = Authentication ( "401-Unauthorized" )
			            buildAuthenticateResponse(httpRequest, httpResponse);
		            }
		        }
		        else
		        {
		            trace("login/password not found in request => '401' (Unauthorized) ");
		            //--- Response = Authentication ( "401-Unauthorized" )
		            buildAuthenticateResponse(httpRequest, httpResponse);
		        }
			}
			else
			{
				trace("URL doesn't start with '" + sAuthURI + "' => '302' Redirect to Authentication URL.");
			    //--- "302" : Redirect to login URL
			    redirectToLoginURL(httpRequest, httpResponse );
			}
		}
	}
	
	//---------------------------------------------------------------------------------------------------------
	private boolean isLoginURL(HttpServletRequest httpRequest) 
	{
		String sAuthURI = httpRequest.getContextPath() + PATH_LOGIN ; // "/context/l/o/g/i/n/"
		String sURI = httpRequest.getRequestURI(); // "/context/path..."
		return sURI.startsWith(sAuthURI) ;
	}
	//---------------------------------------------------------------------------------------------------------
	private String getFullURL(HttpServletRequest request, String sPath) 
	{
	    String sProtocol = request.getScheme() ; // http or https // TODO : force https ? ( get info from properties )
	    String sHost = request.getServerName() ;
	    int iPort = request.getServerPort() ;
	    String sContextPath = request.getContextPath() ;
	    
	    return sProtocol + "://" + sHost + ":" + iPort + sContextPath + sPath ;
	}
	//---------------------------------------------------------------------------------------------------------
	private int getCount(HttpServletRequest request ) //throws IOException, ServletException
	{
	    int iCount = 0 ;
//	    String sQueryString = request.getQueryString() ;
//	    if ( sQueryString != null )
//	    {
//	        if ( sQueryString.startsWith("count=") )
//	        {
//	            
//	        }
//	    }
	    String sCount = request.getParameter(COUNT_PARAM);
	    if ( sCount != null )
	    {
	        iCount = StrUtil.getInt(sCount, 0);
	    }
	    trace("getCount() : return " + iCount);
	    return iCount ;
	}
	//---------------------------------------------------------------------------------------------------------
	private String getToken(HttpServletRequest request ) 
	{
	    String sToken = "";
	    String sURI = request.getRequestURI(); // "/(CONTEXT)/l/o/g/i/n/Txxxxxx/login" ( without QueryString )
		//trace("getToken() : request URI = '" + sURI + "'");
		
		int i = sURI.indexOf(PATH_LOGIN);
		if ( i >= 0 ) // Minimum for "/(CONTEXT)/l/o/g/i/n/"
		{
		    String s = sURI.substring(i+PATH_LOGIN.length()); // "Txxxxxx/login"
			int j = s.indexOf('/');
			if ( j > 0 )
			{
			    sToken = s.substring(0, j); // "Txxxxxxx"
			}
		}
		
		//trace("getToken() : return '" + sToken + "'");
		return sToken ;
	}
	//---------------------------------------------------------------------------------------------------------
	private String getNewToken() 
	{
	    Date date = new Date();	    
	    String sToken = TOKEN_PREFIX + date.getTime() ;
		//trace("getNewToken() : return '" + sToken + "'");
		return sToken ;
	}
	//---------------------------------------------------------------------------------------------------------
	private void redirectToLoginURL(HttpServletRequest request, HttpServletResponse response ) throws IOException //, ServletException
	{
	    String sToken = getNewToken();
//	    String s = fullURL( request, PATH_LOGIN + sToken + "?" + COUNT_PARAM + "=" + iCount ) ;
	    String s = getFullURL( request, PATH_LOGIN + sToken + "/login") ;
		trace("sendRedirect to Login URL -> " + s);
		response.sendRedirect(s);
	}
    //------------------------------------------------------------------------
    /**
     * Build a http response with error code "401-Unauthorized"<br>
     * and header "WWW-Authenticate"
     * @param request
     * @param response
     */
    private void buildAuthenticateResponse(HttpServletRequest request, HttpServletResponse response)
    {
        trace("buildAuthenticateResponse()...");
        //--- Set the "real name" (printed in the navigator dialog-box)
        String sToken = getToken(request) ;
        String sRealmName = TelosysAUTH.getRealmName();
        if ( TelosysAUTH.getRealmName() != null )
        {
            sRealmName = TelosysAUTH.getRealmName() + "(" + sToken + ")";
        }
        else
        {
            sRealmName = request.getContextPath() + "(" + sToken + ")";
        }
        
        response.setHeader("WWW-Authenticate", "Basic realm=\"" + sRealmName + "\"");
        //--- Code Retour HTTP "401-Unauthorized"
        response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
    }

	//---------------------------------------------------------------------------------------------------------
	private void redirectToFirstPage(HttpServletRequest request, HttpServletResponse response, IAppUser appUser ) throws IOException, ServletException
	{
        //--- Try to get a specific First Page for the user
        String sUrlPage = appUser.getFirstPage();
        if (sUrlPage == null)
        {
            //--- Try to get the Application's First Page
            sUrlPage = TelosysAUTH.getFirstPage();
            if (sUrlPage == null)
            {
                //--- Default first page : WebApp root
                //sUrlPage = WebUtil.getRootUrl(request);
                sUrlPage = getFullURL(request, "/") ;
        		trace("No default application first page, use context root '" + sUrlPage + "'");
            }
            else
            {
        		trace("Default application first page = '" + sUrlPage + "'");
            }
        }
        else
        {
    		trace("User first page = '" + sUrlPage + "'");
        }
        if ( sUrlPage.startsWith("http") != true ) // Relative page 
        {
            sUrlPage = getFullURL(request, sUrlPage) ;
        }
		trace("sendRedirect to first page -> " + sUrlPage );
		response.sendRedirect(sUrlPage);
	}
	//---------------------------------------------------------------------------------------------------------
    private LoginUser getUserFromRequest(HttpServletRequest request)
    {
        //--- Get the "Authorization" HTTP header
        String sAuthorization = request.getHeader("Authorization");
        if (sAuthorization == null)
        {
            return null;
        }
        //--- Check the authorization type : "scheme BASIC"
        if (!sAuthorization.toLowerCase().startsWith("basic "))
        {
            return null;
        }
        //--- Get the Base64 string
        String sEncoded = sAuthorization.substring(6).trim();
        //--- Decode the Base64 string and parse it 
        String sDecoded = new String(Base64.decode(sEncoded.getBytes()));
        int iPosiColon = sDecoded.indexOf(':');
        if (iPosiColon < 0)
        {
            return null;
        }
        //--- Get the user login and password
        String sUserLogin = sDecoded.substring(0, iPosiColon).trim();
        String sUserPassword = sDecoded.substring(iPosiColon + 1).trim();

        //--- Retun the LoginUser instance
		LoginUser user = new LoginUser(sUserLogin, sUserPassword) ;        
        return user;
    }
	//---------------------------------------------------------------------------------------------------------
/***
    private IAppUser checkUserAuthentication(LoginUser loginUser)
    {
        //--- Delegate the user validation to the application validator instance
        IAppUser appUser = Auth.validUser(loginUser);
        if (appUser != null) // "login/password" is OK
        {
            log.trace("login/password OK");

            //--- OPEN a SCREEN SESSION for this user
            ScreenSessionManager.openScreenSession(request, loginUser, appUser);

            //--- Try to get a specific First Page for the user
            sUrlPage = appUser.getFirstPage();
            if (sUrlPage == null)
            {
                //--- Try to get the Application's First Page
                sUrlPage = TelosysAUTH.getFirstPage();
                if (sUrlPage == null)
                {
                    //--- Default first page : WebApp root
                    sUrlPage = WebUtil.getRootUrl(request);
                }
            }
            //--- "redirect" to transmit the good URL
            WebUtil.redirect(sUrlPage, response);
        }
        else
        {
            log.trace("invalid login/password");
            //--- Response = Authentication ( "401-Unauthorized" )
            Auth.buildAuthenticateResponse(request, response);
        }
    }
***/

	//---------------------------------------------------------------------------------------------------------
	//---------------------------------------------------------------------------------------------------------
}
