package org.objectweb.telosys.common.data;

import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.vo.FieldComparer;

/**
 * @author Laurent GUERIN
 *  
 */
public abstract class DataSet extends TelosysObject
{
    private final static String DUMMY_OBJECT = "";
    
    private final static int    COL_COUNT_NOT_INITIALIZED = -1;
    
    //-----------------------------------------------------------------------------
    //--- PRIVATE ATTRIBUTES
    //-----------------------------------------------------------------------------
    private LinkedList _vDataRows = null;

    private int        _iRowCount = 0;

    private int        _iColCount = COL_COUNT_NOT_INITIALIZED ;
    
    //-----------------------------------------------------------------------------
    private boolean    _bSorted        = false;

    //private String     _sSortedByField = "";
    private int        _iSortedByCol   = -1;

    private boolean    _bSortOrder     = FieldComparer.ASC;


    /**
     *  
     */
    public DataSet()
    {
        super();
//        //_vDataRows = new Vector(10, 10); // initial capacity = 10, increment = 10
//        _vDataRows = new LinkedList(); 
//        //_vDataRows.add(0, null); // Position 0 : void
//        _vDataRows.add(null); // Position 0 : void
        //newList();
    }

    //-----------------------------------------------------------------------------
    // PRIVATE METHOD(S)
    //-----------------------------------------------------------------------------
    /**
     * Returns the boolean order flag according to the given order in string format
     * @param sOrder
     * @return
     */
    private boolean getSortOrderFlag(String sOrder)
    {
        if (sOrder == null)
        {
            throw new TelosysRuntimeException("DataSet.sort : order parameter is 'null' ");
        }
        String sOrderUC = sOrder.trim().toUpperCase();
        if (sOrderUC.startsWith("ASC"))
        {
            return FieldComparer.ASC;
        }
        else if (sOrderUC.startsWith("DESC"))
        {
            return FieldComparer.DESC;
        }
        else
        {
            throw new TelosysRuntimeException("DataSet.sort : invalid order parameter '" + sOrder + "' ( 'ASC' or 'DESC' expected )");
        }
    }

    /**
     * Sorts the list according to the given parameters
     * @param sFieldName : field to use as sort criteria
     * @param bOrder : sort order flag ( ASC / DESC )
     * @param bFlagIgnoreCase : ignore case flag 
     */
    private void sortList(int iCol, boolean bOrder, boolean bFlagIgnoreCase)
    {
        //--- 1) Remove the DUMMY_OBJECT ( for "base 1" index )
        _vDataRows.remove(0); 
        //--- 2) SORT the list (with only DataRows objects ) 
        Collections.sort(_vDataRows, new DataRowComparer(iCol, bOrder, bFlagIgnoreCase));
        //--- 3) Restore the DUMMY OBJECT at Position 0 ( for "base 1" index )
        _vDataRows.addFirst(DUMMY_OBJECT); 

        _bSorted = true;
        _iSortedByCol = iCol;
        _bSortOrder = bOrder;
    }
    
    //-----------------------------------------------------------------------------
    //--- PROTECTED METHODS
    //-----------------------------------------------------------------------------
    /**
     * Data set initialization, must be called by subclasses before each load
     */
    protected void initBeforeLoad()
    {
        if ( _vDataRows != null )
        {
            _vDataRows.clear(); // just for GC 
        }
        _vDataRows = new LinkedList(); 
        _vDataRows.add(DUMMY_OBJECT); // DUMMY OBJECT at Position 0 ( for "base 1" index )
        _iColCount = COL_COUNT_NOT_INITIALIZED ;
        _iRowCount = 0;
    }
    //-----------------------------------------------------------------------------
    /**
     * Replaces the DataRow at the given position by a new one
     * @param iRow index of row to replace ( 1 to N )
     * @param dataRow the DataRow to be stored at the specified position
     * @return the DataRow previously at the specified position
     */
    protected DataRow replaceRow(int iRow, DataRow dataRow)
    {
        if ( _vDataRows == null )
        {
            throw new TelosysRuntimeException("DataSet replaceRow("+iRow+",DataRow) : the DataSet is not initialized !");
        }
        //--- Is the row position valid ?
        if (iRow < 1 || iRow > _iRowCount)
        {
            throw new TelosysRuntimeException("DataSet replaceRow("+iRow+",DataRow) : invalid row position !");
        }
        if ( dataRow == null )
        {
            throw new TelosysRuntimeException("DataSet replaceRow("+iRow+",null) : data row is null !");
        }
        //--- Is the column count compatible with the DataSet ?
        if ( dataRow.getSize() != _iColCount )
        {
            throw new TelosysRuntimeException("DataSet replaceRow("+iRow+",DataRow) : invalid data row size (" 
                    + dataRow.getSize() + " column(s) : " + _iColCount + " expected )");
        }
        return (DataRow) _vDataRows.set(iRow, dataRow);
    }
    //-----------------------------------------------------------------------------
    /**
     * Adds a DataRow at the end of the DataSet
     * @param dataRow the DataRow to add
     * @return
     */
    protected int addRow(DataRow dataRow)
    {
        if ( _vDataRows == null )
        {
            throw new TelosysRuntimeException("DataSet addRow(DataRow) : the DataSet is not initialized !");
        }
        //--- The first row added determined the number of columns
        if ( _iColCount == COL_COUNT_NOT_INITIALIZED )
        {
            _iColCount = dataRow.getSize();
        }
        //--- Add the row
        _vDataRows.add(dataRow);
        _iRowCount++;
        return _iRowCount;
    }
    //-----------------------------------------------------------------------------
    /**
     * Adds a new void DataRow at the end of the DataSet
     * @return
     */
    protected int addRow()
    {
        if ( _vDataRows == null )
        {
            throw new TelosysRuntimeException("DataSet addRow() : the DataSet is not initialized !");
        }
        return addRow( new DataRow(_iColCount)) ;
    }
    
    //-----------------------------------------------------------------------------
    //--- PUBLIC ABSTRACT METHODS
    //-----------------------------------------------------------------------------
    /**
     * Loads the DataSet using its definition
     * 
     * @return
     * @throws Exception
     */
    public abstract int load() throws Exception;

    //-----------------------------------------------------------------------------
    /**
     * Loads the DataSet using its definition and the given object
     * 
     * @param obj
     *            useful parameter for the load operation ( cannot be null )<br>
     *            e.g. with SQLDataSet : Connection, DatabaseSession, ... 
     * @return
     * @throws Exception
     */
    public abstract int load(Object obj) throws Exception;

    //-----------------------------------------------------------------------------
    //--- PUBLIC METHODS
    //-----------------------------------------------------------------------------
    /**
     * Returns the number of rows
     * 
     * @return
     */
    public int getRowCount()
    {
        return _iRowCount;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns 'true' if the DataSet contains no elements.
     * 
     * @return
     */
    public boolean isEmpty()
    {
        return (_iRowCount < 1 ? true : false);
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns an iterator over the 'datarows' in this DataSet
     * 
     * @return
     */
    public Iterator iterator()
    {
        return new DataSetIterator(this);
    }

    //-----------------------------------------------------------------------------
    public DataRow getDataRow(int iRow)
    {
        if (iRow < 1 || iRow > _iRowCount)
        {
            return null;
        }
        else
        {
            return (DataRow) _vDataRows.get(iRow);
        }
    }

//    //-----------------------------------------------------------------------------
//    public void toXml(StringBuffer sb)
//    {
//        toXml(sb, "list");
//    }

//    //-----------------------------------------------------------------------------
//    public void toXml(StringBuffer sb, String sTagList)
//    {
//        String sCRLF = "\n";
//
//        sb.append("<" + sTagList + ">" + sCRLF);
//        DataRow dr = null;
//        int iSize = 0;
//        for ( int iRow = 1 ; iRow <= _iRowCount ; iRow++ )
//        {
//            dr = (DataRow) _vDataRows.get(iRow);
//            if (dr != null)
//            {
//                iSize = dr.getSize();
//                Object obj = null;
//                sb.append("<row");
//                //--- For each column in the DataRow
//                for ( int iCol = 1 ; iCol <= iSize ; iCol++ )
//                {
//                    obj = dr.getObject(iCol);
//                    if (obj != null)
//                    {
//                        sb.append(" c" + iCol + "=\"" + XmlUtil.xmlString(obj.toString()) + "\"");
//                    }
//                }
//                sb.append(" />" + sCRLF);
//            }
//        }
//        sb.append("</" + sTagList + ">" + sCRLF);
//    }

    //-----------------------------------------------------------------------------
    // Get Cell value with different types
    //-----------------------------------------------------------------------------
    public Object getObject(int iRow, int iCol)
    {
        DataRow dr = getDataRow(iRow);
        if (dr != null)
        {
            return dr.getObject(iCol);
        }
        return null;
    }

    //-----------------------------------------------------------------------------
    public String getString(int iRow, int iCol)
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getString(iCol) : null);
    }

    //-----------------------------------------------------------------------------
    public String getString(int iRow, int iCol, String sNullValue)
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getString(iCol, sNullValue) : null);
    }

    //-----------------------------------------------------------------------------
    public java.util.Date getDate(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getDate(iCol) : null);
    }

    //-----------------------------------------------------------------------------
    public java.sql.Date getSqlDate(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getSqlDate(iCol) : null);
    }

    //-----------------------------------------------------------------------------
    public boolean getBoolean(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getBoolean(iCol) : false);
    }

    //-----------------------------------------------------------------------------
    public byte getByte(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getByte(iCol) : 0);
    }

    //-----------------------------------------------------------------------------
    public int getInt(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getInt(iCol) : 0);
    }

    //-----------------------------------------------------------------------------
    public short getShort(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getShort(iCol) : 0);
    }

    //-----------------------------------------------------------------------------
    public long getLong(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getLong(iCol) : 0);
    }

    //-----------------------------------------------------------------------------
    public float getFloat(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getFloat(iCol) : 0);
    }

    //-----------------------------------------------------------------------------
    public double getDouble(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getDouble(iCol) : 0);
    }
    //-----------------------------------------------------------------------------
    
    
    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
    /**
     * Returns the field name used as current sort criteria
     * 
     * @return : field name, or "" if the list is not sorted
     */
//    public String getSortField()
//    {
//        if (_bSorted && (_sSortedByField != null))
//        {
//            return _sSortedByField;
//        }
//        return "";
//    }
    public int getSortColumn()
    {
        if (_bSorted )
        {
            return _iSortedByCol;
        }
        return -1;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns true if the list is currently sorted by the given field
     * 
     * @param sFieldName
     * @return true/false
     */
//    public boolean isSortedBy(String sFieldName)
//    {
//        if (sFieldName == null)
//        {
//            return false;
//        }
//        if (_bSorted && (_sSortedByField != null))
//        {
//            if (_sSortedByField.equalsIgnoreCase(sFieldName.trim()))
//            {
//                return true;
//            }
//        }
//        return false;
//    }
    public boolean isSortedBy(int iCol)
    {
        if (_bSorted && (_iSortedByCol == iCol) )
        {
            return true;
        }
        return false;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns true if the list is currently sorted in ascending order
     * @return
     */
    public boolean isSortedInAscendingOrder()
    {
        if (_bSorted && (_bSortOrder == FieldComparer.ASC))
        {
            return true;
        }
        return false;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns true if the list is currently sorted in descending order
     * @return
     */
    public boolean isSortedInDescendingOrder()
    {
        if (_bSorted && (_bSortOrder == FieldComparer.DESC))
        {
            return true;
        }
        return false;
    }

    //-----------------------------------------------------------------------------
    /**
     * Sort the list using the given field as criteria (case sensitive) in ascending order
     * 
     * @param sFieldName :
     *            the name of the field to compare
     */
    //public void sort(String sFieldName)
    public void sort(int iCol)
    {
        sortList(iCol, FieldComparer.ASC, FieldComparer.DO_NOT_IGNORE_CASE);
    }

    //-----------------------------------------------------------------------------
    /**
     * Sort the list using the given field as criteria (case sensitive) and the specified order ( ascending or descending )
     * 
     * @param sFieldName :
     *            the name of the field to compare
     * @param sOrder :
     *            'ASC' = ascending order , 'DESC' = descending order
     */
    //public void sort(String sFieldName, String sOrder)
    public void sort(int iCol, String sOrder)
    {
        boolean bOrder = getSortOrderFlag(sOrder);
        sortList(iCol, bOrder, FieldComparer.DO_NOT_IGNORE_CASE);
    }

    //-----------------------------------------------------------------------------
    /**
     * Sort the list using the given field as criteria ( ignoring case ) in ascending order
     * 
     * @param sFieldName :
     *            the name of the field to compare
     */
    //public void sortIgnoreCase(String sFieldName)
    public void sortIgnoreCase(int iCol)
    {
        sortList(iCol, FieldComparer.ASC, FieldComparer.IGNORE_CASE);
    }

    //-----------------------------------------------------------------------------
    /**
     * Sort the list using the given field as criteria ( ignoring case ) and the specified order ( ascending or descending )
     * 
     * @param sFieldName :
     *            the name of the field to compare
     * @param sOrder :
     *            'ASC' = ascending order , 'DESC' = descending order
     */
    //public void sortIgnoreCase(String sFieldName, String sOrder)
    public void sortIgnoreCase(int iCol, String sOrder)
    {
        boolean bOrder = getSortOrderFlag(sOrder);
        sortList(iCol, bOrder, FieldComparer.IGNORE_CASE);
    }
    
}