package org.objectweb.telosys.dal.sql;

import java.io.InputStream;
import java.sql.Connection;
import java.util.Properties;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.dal.dao.DatabaseSession;
import org.objectweb.telosys.dal.dao.StandardDatabaseSession;
import org.objectweb.telosys.util.StrUtil;
import org.objectweb.telosys.util.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Unique (static) access point to get databases connections
 * 
 * @author Laurent GUERIN
 *  
 */
public final class ConnectionManager
{
    private static final TelosysClassLogger log               = new TelosysClassLogger(ConnectionManager.class);

    private static String                   $sConfigFileName  = null;

    private static ConnectionProvider[]     $conProviders     = null;

    private static int                      $iDefaultDatabase = 0;

    //---------------------------------------------------------------------------
    private static Properties getProperties(Element elem)
    {
        NodeList nodes = elem.getElementsByTagName("property");
        Properties prop = null;
        if (nodes.getLength() > 0)
        {
            prop = new Properties();
            for ( int iProp = 0 ; iProp < nodes.getLength() ; iProp++ )
            {
                Element eltProperty = (Element) nodes.item(iProp);
                String sPropName = eltProperty.getAttribute("name");
                String sPropValue = eltProperty.getAttribute("value");
                prop.put(sPropName, sPropValue);
                log.trace("     " + iProp + " -> property : " + sPropName + "=" + sPropValue);
            }
        }
        return prop;
    }

    //---------------------------------------------------------------------------
    private static boolean isValidDataSource(String sDataSource)
    {
        if (sDataSource != null)
        {
            if (sDataSource.trim().length() > 0)
            {
                return true;
            }
        }
        return false;
    }

    //---------------------------------------------------------------------------
    /**
     * Creates a Connection Provider according with the "db" configuration tag parameters
     * 
     * @param sDbId
     * @param sDbName
     * @param sDataSource
     * @param sDbDriver
     * @param sDbUrl
     * @param props
     * @param sDbIsolationLevel
     * @param sDbPoolSize
     * @return the ConnectionProvider created
     */
    private static ConnectionProvider createConnectionProvider(String sDbId, String sDbName, String sDataSource, String sDbDriver, String sDbUrl, Properties props, String sDbIsolationLevel, String sDbPoolSize)
    {
        log.trace("createConnectionProvider : id=" + sDbId + " name=" + sDbName + " datasource=" + sDataSource + " driver=" + sDbDriver + " url=" + sDbUrl + " poolSize=" + sDbPoolSize + " isolation level=" + sDbIsolationLevel);

        ConnectionProvider cp = null;
        if (isValidDataSource(sDataSource))
        {
            //--- Connection Provider = standard JDBC DataSource
            log.info("DB " + sDbId + " (" + sDbName + ") : Create ConnectionProvider : DataSource '" + sDataSource + "' ...");
            try
            {
                cp = new SqlDataSource(sDbName, sDataSource);
            } catch (TelosysException e)
            {
                log.error("Cannot create SqlDataSource : TelosysException : " + e.getMessage());
                return null;
            }
        }
        else
        {
            log.trace("DB " + sDbId + " (" + sDbName + ") driver    = " + sDbDriver);
            log.trace("DB " + sDbId + " (" + sDbName + ") url       = " + sDbUrl);

            int iDbPoolSize = StrUtil.getInt(sDbPoolSize, 0);
            log.trace("DB " + sDbId + " (" + sDbName + ") pool size = " + sDbPoolSize + "(" + iDbPoolSize + ")");

            int iDbIsolationLevel = SqlTool.getIsolationLevel(sDbIsolationLevel, Connection.TRANSACTION_READ_COMMITTED);
            log.trace("DB " + sDbId + " (" + sDbName + ") isolation level = " + sDbIsolationLevel + "(" + iDbIsolationLevel + ")");

            if (iDbPoolSize > 1)
            {
                //--- Connection Pool
                log.info("DB " + sDbId + " (" + sDbName + ") : Create ConnectionProvider : Telosys Connections Pool (size=" + iDbPoolSize + ") ...");
                cp = new SqlConnectionPool(sDbName, sDbDriver, sDbUrl, props, iDbIsolationLevel, iDbPoolSize);
            }
            else
            {
                //--- No pool ( simple Connection Factory )
                log.info("DB " + sDbId + " (" + sDbName + ") : Create ConnectionProvider : Telosys simple Connection Factory ...");
                cp = new SqlConnectionFactory(sDbName, sDbDriver, sDbUrl, props, iDbIsolationLevel);
            }
        }
        return cp;
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Connection manager initialization <br>
     * Load the 'dbcongif' XML file and create a connection provider for each database definition
     * 
     * @param sFileName :
     *            dbconfig file name
     * @return
     */
    public static boolean init(String sFileName)
    {
        log.info("init(" + sFileName + ")");        
        if (sFileName == null)
        {
            log.error("init(" + sFileName + ") file name parameter is null" );
            return false ;
        }
        //--- Keep the dbconfig file name
        $sConfigFileName = sFileName;
        
        //Document document = XmlUtil.loadFile(sFileName);
        Document document = null;
        try
        {
            document = XmlUtil.parse(sFileName);
        } catch (TelosysException e)
        {
            log.error("init(" + sFileName + ") cannot parse. TelosysException : " + e.getMessage() );
            return false ;
        }
        
        return init( document );
    }
    
    //---------------------------------------------------------------------------------------------
    public static boolean init(InputStream is)
    {
        log.info("init(InputStream)");        
        if (is == null)
        {
            log.error("init(InputStream) InputStream parameter is null" );
            return false ;
        }
        //--- Keep the dbconfig file name
        $sConfigFileName = null ;
        
        //Document document = XmlUtil.loadFile(sFileName);
        Document document = null;
        try
        {
            document = XmlUtil.parse(is);
        } catch (TelosysException e)
        {
            log.error("init(InputStream) cannot parse. TelosysException : " + e.getMessage() );
            return false ;
        }
        
        return init( document );
    }
    
    //---------------------------------------------------------------------------------------------
    private static boolean init( Document document )
    {
        if (document == null)
        {
            return false ;
        }
        
        Element root = document.getDocumentElement();

        //--- Number of Databases = Maximum Database ID + 1 ( 10 if not set, cannot exceed 100)
        String sMaxId = root.getAttribute("maxId");
        int iMaxId = StrUtil.getInt(sMaxId, 10);
        if (iMaxId > 100)
        {
            log.error("maxId = " + iMaxId + " : maxId > 100 => reduce to 100 !");
            iMaxId = 100;
        }
        int iNbDatabases = iMaxId + 1;
        log.trace("Maximum database Id = '" + sMaxId + "' ( int : " + iMaxId + ")");
        log.trace("Number of database  = " + iNbDatabases + " ( maxId + 1 ) ");

        //--- Default Database ID ( 0 if not set )
        String sDefaultId = root.getAttribute("defaultId");
        $iDefaultDatabase = StrUtil.getInt(sDefaultId, 0);
        if ($iDefaultDatabase < 0 || $iDefaultDatabase >= iNbDatabases)
        {
            log.error("Invalid default database (" + $iDefaultDatabase + ") : set to 0 ");
            $iDefaultDatabase = 0;
        }
        log.trace("Default database Id = '" + $iDefaultDatabase + "' ( int : " + $iDefaultDatabase + ")");

        //--- Array of 'N' providers
        $conProviders = new ConnectionProvider[iNbDatabases];
        log.info("Ready to store " + iNbDatabases + " connection providers ( ID from 0 to " + iMaxId + " )");

        //--- Databases list
        NodeList nodes = document.getElementsByTagName("db");
        int nbDbNodes = nodes.getLength();

        //--- For each database defined in the XML file ...
        for ( int i = 0 ; i < nbDbNodes ; i++ )
        {
            //--- Tag "bd" ( get the attributes )
            Element eltBase = (Element) nodes.item(i);
            if (eltBase == null)
            {
                log.error("XML 'db' node is null ( Node " + i + " ) !");
            }
            else
            {

                String sDbId = eltBase.getAttribute("id");
                int iDbId = StrUtil.getInt(sDbId, -1);
                String sDbName = eltBase.getAttribute("name");

                //--- Database managed by a DataSource
                String sDataSource = eltBase.getAttribute("datasource");

                //--- Database managed by Telosys-DAL
                String sDbDriver = eltBase.getAttribute("driver");
                String sDbUrl = eltBase.getAttribute("url");
                String sDbIsolationLevel = eltBase.getAttribute("isolationLevel");
                //int iDbIsolationLevel = SqlTool.getIsolationLevel(sDbIsolationLevel, Connection.TRANSACTION_READ_COMMITTED);
                String sDbPoolSize = eltBase.getAttribute("poolSize");
                //int iDbPoolSize = StrUtil.getInt(sDbPoolSize, 0);

                //--- Tags "property"
                Properties props = getProperties(eltBase);

                //--- Connection Provider creation :
                ConnectionProvider cp = createConnectionProvider(sDbId, sDbName, sDataSource, sDbDriver, sDbUrl, props, sDbIsolationLevel, sDbPoolSize);
                if ( cp != null )
                {
                    log.info("DB " + sDbId + " (" + sDbName + ") : ConnectionProvider created.");                        
                }
                //--- Store provider in the array
                if (iDbId >= 0 && iDbId < $conProviders.length)
                {
                    if ($conProviders[iDbId] == null)
                    {
                        if (cp != null)
                        {
                            $conProviders[iDbId] = cp;
                            log.info("DB " + sDbId + " (" + sDbName + ") : ConnectionProvider ready.");
                        }
                        else
                        {
                            log.error("DB " + sDbId + " (" + sDbName + ") : ConnectionProvider is NULL !");
                        }
                    }
                    else
                    {
                        log.error("Database ID " + iDbId + " is already used : cannot register this database ! ");
                    }
                }
                else
                {
                    log.error("Invalid Database ID ( " + iDbId + " ) : cannot register this database ! ");
                }
            }
        }
        return true ;
    }

    //---------------------------------------------------------------------------------------------
    public static DatabaseSession getSession() throws TelosysException
    {
        return getSession($iDefaultDatabase);
    }

    //---------------------------------------------------------------------------------------------
    public static DatabaseSession getSession(int iDatabaseId) throws TelosysException
    {
        Connection con = getConnection(iDatabaseId);
        return new StandardDatabaseSession(iDatabaseId, con);
    }

    //---------------------------------------------------------------------------------------------
//    /**
//     * Returns the configuration file name used to initialized the Connection Manager
//     * 
//     * @return the file name (or null)
//     */
//    public static String getConfigFileName()
//    {
//        return $sConfigFileName;
//    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns the default database id
     * 
     * @return
     */
    public static int getDefaultDatabase()
    {
        return $iDefaultDatabase;
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns the number of databases managed by the connection manager
     * 
     * @return
     */
    public static int getNumberOfDatabases()
    {
        if ($conProviders != null)
        {
            return $conProviders.length;
        }
        return 0;
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns a connection for the default database
     * 
     * @return
     * @throws TelosysException
     */
    public static Connection getConnection() throws TelosysException
    {
        return getConnection($iDefaultDatabase);
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns a connection for the given database id
     * 
     * @param iBaseId
     * @return
     * @throws TelosysException
     */
    public static Connection getConnection(int iBaseId) throws TelosysException
    {
        //SqlConnectionProvider cp = getConnectionProvider(iBaseId);
        ConnectionProvider cp = getConnectionProvider(iBaseId);
        if (cp != null)
        {
            try
            {
                return cp.getConnection();
            } catch (Exception ex)
            {
                throw new TelosysException("getConnection(" + iBaseId + ") : Exception ! " + ex);
            }
        }
        else
        {
            throw new TelosysException("getConnection(" + iBaseId + ") : No ConnectionProvider for this DB id !");
        }
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns the default database connection provider
     * 
     * @return
     */
    protected static ConnectionProvider getConnectionProvider()
    {
        return getConnectionProvider($iDefaultDatabase);
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Return the connection provider for the given database id
     * 
     * @param iBaseId
     * @return the connection provider (or null if no provider for the given DB id)
     */
    protected static ConnectionProvider getConnectionProvider(int iBaseId)
    {
        if ($conProviders != null)
        {
            if (iBaseId >= 0 && iBaseId <= $conProviders.length)
            {
                if ($conProviders[iBaseId] != null)
                {
                    return $conProviders[iBaseId];
                }
                else
                {
                    log.error("getConnectionProvider(" + iBaseId + ") : no connection provider for this database ID ! ");
                    return null;
                }
            }
            else
            {
                log.error("getConnectionProvider(" + iBaseId + ") : invalid database ID ! ");
                return null;
            }
        }
        else
        {
            log.error("getConnectionProvider(" + iBaseId + ") : connection providers not initialized ! ");
            return null;
        }
    }
    //---------------------------------------------------------------------------------------------
}