package org.objectweb.telosys.screen.core;

import java.util.Map;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.dal.dao.DatabaseSession;
import org.objectweb.telosys.dal.dao.DummyDatabaseSession;
import org.objectweb.telosys.dal.sql.ConnectionManager;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenResponse;

/**
 * @author Laurent GUERIN
 *  
 */
public abstract class StandardScreenDataAccessor extends TelosysObject implements ScreenDataAccessor
{
    protected static final int  DEFAULT_DATABASE = -1;

    protected static final int  NO_DATABASE      = -99;
    
    private static final String ACTION_INIT      = "init";
    
    private static final String ACTION_LOAD      = "load";

    private static final String ACTION_SAVE      = "save";

    private static final String ACTION_DELETE    = "delete";

    private static final String ACTION_INSERT    = "insert";

    private static final String ACTION_UPDATE    = "update";

    /**
     * The ScreenData class ( used only to check parameters class )
     */
    private Class               _screenDataClass = null;

    /**
     * The database ID (used to get a database connection)
     */
    private int                 _iDatabaseId     = 0;

    /**
     * Constructor for a Data Accessor using the default database
     * 
     * @param screenDataClass :
     *            the ScreenData class
     */
    public StandardScreenDataAccessor(Class screenDataClass)
    {
        super();
        _screenDataClass = screenDataClass;
        _iDatabaseId = DEFAULT_DATABASE;
    }

    /**
     * Constructor for a Data Accessor using a specific database
     * 
     * @param screenDataClass :
     *            the ScreenData class
     * @param iDatabaseId :
     *            the id of the Database to use (to get a connection)
     */
    public StandardScreenDataAccessor(Class screenDataClass, int iDatabaseId)
    {
        super();
        _screenDataClass = screenDataClass;
        _iDatabaseId = iDatabaseId;
    }

    //=========================================================================================================
    // Abstract methods ( to be implemented in the concrete Screen DAO )
    //=========================================================================================================

    public abstract void init(ScreenData screenData, ScreenContext screenContext, Map param, DatabaseSession dbSession) throws TelosysException ;
    
    public abstract void load(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException;

    public abstract void save(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException;

    public abstract void insert(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException;

    public abstract void update(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException;

    public abstract void delete(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException;

    //=========================================================================================================
    // Check methods
    //=========================================================================================================
    /**
     * Checks the Database session validity
     * 
     * @param dbSession
     * @param sAction
     * @throws TelosysException
     */
    private void checkDatabaseSession(DatabaseSession dbSession, String sAction) throws TelosysException
    {
        //        if (dbSession == null && _iDatabaseId != NO_DATABASE )
        //        {
        //            String sClassName = this.getClass().getName();
        //            throw new TelosysException("ScreenDAO (" + sClassName + ") : " + sAction + " : No database session (dbSession == NULL) !");
        //        }
        if (dbSession == null)
        {
            String sClassName = this.getClass().getName();
            throw new TelosysException("ScreenDataAccessor (" + sClassName + ") : " + sAction + " : No database session (dbSession == NULL) !");
        }
    }

    /**
     * Checks the ScreenData instance validity
     * 
     * @param screenData
     * @param sAction
     * @throws TelosysException
     */
    private void checkScreenData(ScreenData screenData, String sAction) throws TelosysException
    {
        String sClassName = this.getClass().getName();
        if (screenData == null)
        {
            throw new TelosysException("ScreenDataAccessor (" + sClassName + ") : " + sAction + " : No screen data (data == NULL) !");
        }
        else
        {
            if (_screenDataClass != null)
            {
                if (!_screenDataClass.isInstance(screenData))
                {
                    throw new TelosysException("ScreenDataAccessor (" + sClassName + ") : " + sAction + " : Screen data is not an instance of '" + _screenDataClass.getName() + "' !");
                }
            }
            else
            {
                throw new TelosysException("ScreenDataAccessor (" + sClassName + ") : " + sAction + " : Unknown ScreenData type ( not specified in the constructor ) !");
            }
        }
    }

    //=========================================================================================================
    // Generic methods ( interface implementation )
    //=========================================================================================================
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenDataAccessor#getDatabaseSession()
     */
    public DatabaseSession getDatabaseSession() throws TelosysException
    {
        if (_iDatabaseId == NO_DATABASE)
        {
            //--- Return a dummy database session
            trace("getDatabaseSession() : return a 'Dummy Session' ");
            //return new DummyDatabaseSession();
            return DummyDatabaseSession.getInstance();
        }
        else
        {
            if (_iDatabaseId == DEFAULT_DATABASE)
            {
                //--- Return a database session for the default database
                trace("getDatabaseSession() : return a 'Real Session' from default database ");
                return ConnectionManager.getSession();
            }
            else
            {
                //--- Return a database session for the specific database
                trace("getDatabaseSession() : return a 'Real Session' from database " + _iDatabaseId);
                return ConnectionManager.getSession(_iDatabaseId);
            }
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenDataAccessor#doInit(org.objectweb.telosys.screen.core.ScreenData, org.objectweb.telosys.screen.core.ScreenContext, java.util.Map, org.objectweb.telosys.dal.dao.DatabaseSession)
     */
    public void doInit(ScreenData screenData, ScreenContext screenContext, Map param, DatabaseSession dbSession) throws TelosysException
    {
        checkScreenData(screenData, ACTION_INIT);
        checkDatabaseSession(dbSession, ACTION_INIT);
        init(screenData, screenContext, param, dbSession);
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenDataAccessor#doLoad(org.objectweb.telosys.screen.core.ScreenData, org.objectweb.telosys.screen.core.ScreenContext, org.objectweb.telosys.rpl.ScreenRequest, org.objectweb.telosys.rpl.ScreenResponse, org.objectweb.telosys.dal.dao.DatabaseSession)
     */
    public void doLoad(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException
    {
        checkScreenData(screenData, ACTION_LOAD);
        checkDatabaseSession(dbSession, ACTION_LOAD);
        load(screenData, screenContext, request, response, dbSession);
    }

    public void doSave(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException
    {
        checkScreenData(screenData, ACTION_SAVE);
        checkDatabaseSession(dbSession, ACTION_SAVE);
        save(screenData, screenContext, request, response, dbSession);
    }

    public void doInsert(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException
    {
        checkScreenData(screenData, ACTION_INSERT);
        checkDatabaseSession(dbSession, ACTION_INSERT);
        insert(screenData, screenContext, request, response, dbSession);
    }

    public void doUpdate(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException
    {
        checkScreenData(screenData, ACTION_UPDATE);
        checkDatabaseSession(dbSession, ACTION_UPDATE);
        update(screenData, screenContext, request, response, dbSession);
    }

    public void doDelete(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException
    {
        checkScreenData(screenData, ACTION_DELETE);
        checkDatabaseSession(dbSession, ACTION_DELETE);
        delete(screenData, screenContext, request, response, dbSession);
    }
}