package org.objectweb.telosys.util;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * @author Laurent GUERIN
 *
 */
public class BeanUtil
{
    private static final TelosysClassLogger $log = new TelosysClassLogger(BeanUtil.class);

    //----------------------------------------------------------------------------------------------
    private static Method findMethod(Class beanClass, String sMethodName)
    {
        Method meth = null;
        try
        {
            meth = beanClass.getMethod(sMethodName, null);
        } catch (SecurityException e)
        {
            throw new TelosysRuntimeException("findMethod : security violation ", e);
            //return null;
        } catch (NoSuchMethodException e)
        {
            return null; // Not found
        }
        return meth;
    }

    //----------------------------------------------------------------------------------------------
    private static Object callGetterMethod(Method method, Object oBean)
    {
        try
        {
            return method.invoke(oBean, null);
        } catch (IllegalArgumentException e)
        {
            throw new TelosysRuntimeException("callGetterMethod : IllegalArgumentException ", e);
            //return null;
        } catch (IllegalAccessException e)
        {
            throw new TelosysRuntimeException("callGetterMethod : IllegalAccessException ", e);
            //return null;
        } catch (InvocationTargetException e)
        {
            throw new TelosysRuntimeException("callGetterMethod : InvocationTargetException ", e);
            //return null;
        }
    }

    //----------------------------------------------------------------------------------------------
    /**
     * Returns the value of the given bean field using reflection
     * @param oBean : the bean instance 
     * @param sFieldName : the field name 
     * @return : the value found 
     * @throws TelosysException if the field name is unknown or if arguments are invalid
     */
    public static Object getFieldValue(Object oBean, String sFieldName) throws TelosysException
    {
        if (oBean == null)
        {
            throw new TelosysException("getFieldValue : bean instance is null ");
        }
        if (sFieldName == null)
        {
            throw new TelosysException("getFieldValue : field name is null ");
        }

        //--- Build the "method field name" (1rst char Upper Case)
        String sFieldName2 = sFieldName.trim();
        char cFirstChar = sFieldName2.charAt(0);
        if (Character.isLowerCase(cFirstChar))
        {
            String s1 = sFieldName2.substring(0, 1);
            String s2 = sFieldName2.substring(1);
            sFieldName2 = s1.toUpperCase() + s2;
        }

        Class beanClass = oBean.getClass();
        Method getter = null;
        //--- Try to find a "getXxxx" getter method
        getter = findMethod(beanClass, "get" + sFieldName2);
        if (getter != null)
        {
            return callGetterMethod(getter, oBean);
        }
        else
        {
            //--- Try to find a "isXxxx" getter method
            getter = findMethod(beanClass, "is" + sFieldName2);
            if (getter != null)
            {
//                System.out.println("get" + sFieldName2 + " : found ");
                return callGetterMethod(getter, oBean);
            }
            else
            {
                //--- Unknown field
                throw new TelosysException("getFieldValue : unknown field ");
            }
        }
    }

    private static String trimNotVoidOrNull(String s) 
    {
        if ( s != null )
        {
            String s2 = s.trim();
            if ( s2.length() > 0)
            {
                return s2;
            }
        }        
        return null ;
    }
    
    //----------------------------------------------------------------------------------------------
    /**
     * Returns the "bean name" from the variable name ( in standard "dot notation" ) 
     * @param sName : the variable name to parse ( 'bean.attribute' )
     * @return : the "bean name" part : 'aaa' for 'aaa.bbb', 'aaa' for 'aaa' or null for '.aaa'
     */
    public static String getBeanName(String sName) 
    {
        if ( sName == null )
        {
            return null ;
        }
        if ( sName.trim().length() == 0 )
        {
            return null ;
        }
        int iDotIndex = sName.indexOf('.');
        if (iDotIndex > 0)
        {
            //--- There's at least a "." in the name ( and not at the first position )
            String[] tokens = sName.split("\\.");
            if (tokens.length == 2) // Exactly 2 parts : "bean.attribute"
            {
                //--- Return the first part : "bean.attribute" => return "bean"
                return trimNotVoidOrNull( tokens[0] );
            }
            else // More than 2 parts : "aa.bb.cc" or more
            {
                return null ;
            }
        }
        else
        {
            if (iDotIndex < 0) // No '.' in the name => return all the name
            {
                return trimNotVoidOrNull(sName);
            }
        }
        // Other cases : "aaa.bbb.ccc.xxx", "aa", ".aa" => return null 
        return null ;
    }
    
    //----------------------------------------------------------------------------------------------
    /**
     * @param sName : the variable name to parse ( 'bean.attribute', 'bean', '.attribute' )
     * @return : the "attribute name" part ( 'attr' for 'bean.attr' or '.attr' )<br>
     *  or null for 'bean', 'aa.bb.cc', ''
     */
    public static String getAttributeName(String sName) 
    {
        if ( sName == null )
        {
            return null ;
        }
        if ( sName.trim().length() == 0 )
        {
            return null ;
        }
        String sFieldName = null;
        int iDotIndex = sName.indexOf('.');
        if (iDotIndex > 0)
        {
            //--- There's at least a "." in the name ( not at the begining : "bean.attribute" )
            String[] tokens = sName.split("\\.");
            if (tokens.length == 2) // Exactly 2 parts : "bean.attribute"
            {
                //--- Return the 2nd part : "bean.attribute" => return "attribute"
                sFieldName = tokens[1];
            }
            else // More than 2 parts : "aa.bb.cc" or more
            {
                return null ;
            }
        }
        else
        {
            // No '.' or '.' is at the first position => no bean name
            if (iDotIndex == 0) // One '.' at the beginning : ".attribute"
            {
                //--- Remove the '.' at the first position
                sFieldName = sName.substring(1);
            }
            else // No '.'
            {
                //--- Take it "as is"
                //sFieldName = sName;
                return null ;
            }
        }    
        return trimNotVoidOrNull(sFieldName);
    }
}
